/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.sensor;

import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.Arrays;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.Palette;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.Lookup;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 * Node which represents sensor files
 *
 * @author Merijn Zengers
 */
public class SensorFileNode extends FilterNode {

    private Node original;
    private Layer layer;

    /**
     * <p> Creates new SensorFile node Will place the provided layer and the {@link SensorGrid}
     * returned by  {@link SensorDataObject#getSensorGrid()} in it's lookup. </p>
     * <p>adds {@link PaletteChangeListener} on provided {@link SensorDataObject}
     * to listen to {@link PropertyChangeEvent} of {@link PropertyChangeEvent#propertyName} {@link SensorDataObject#PROP_PALETTE_CHANGED}
     * </p>
     *
     * @param original
     * @param sensorDataObject
     * @param layer
     */
    public SensorFileNode(Node original, SensorDataObject sensorDataObject, Layer layer) {
        super(original, FilterNode.Children.LEAF, new ProxyLookup(new Lookup[]{original.getLookup(), Lookups.fixed(layer, sensorDataObject.getSensorGrid())}));
        this.original = original;
        this.layer = layer;
        sensorDataObject.addPropertyChangeListener(new PaletteChangeListener());
    }

    /**
     * Adds a {@link ChangePaletteAction} to the actions from the original node
     *
     * @param arg0
     * @return
     */
    @Override
    public Action[] getActions(boolean arg0) {
        Action[] addedActions = {new ChangePaletteAction()};

        Action[] originalActions = original.getActions(arg0);
        originalActions[0].setEnabled(true);
        Action[] actions = Arrays.copyOf(originalActions, originalActions.length + addedActions.length);

        int counter = originalActions.length;
        for (Action addedAction : addedActions) {
            actions[counter] = addedAction;
        }
        return actions;
    }

    /**
     * This implementation only calls clipboardCopy supposing that copy to
     * clipboard and copy by d'n'd are similar.
     *
     * @return transferable to represent this node during a drag
     * @exception IOException when the cut cannot be performed
     */
    @Override
    public Transferable drag() throws IOException {
        return clipboardCopy();
    }

    /**
     * Copy this node to the clipboard.
     *
     * @return {@link org.openide.util.datatransfer.ExTransferable.Single} with
     * one copy flavor
     * @throws IOException if it could not copy
     * @see NodeTransfer
     */
    @Override
    public Transferable clipboardCopy() throws IOException {
        return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    /**
     * Listens to PropertyChanges of type {@link SensorDataObject.PROP_PALETTE_CHANGED}
     * Sets the pallete of {@link SensorFileNode#layer} to the {@link PropertyChangeEvent#newValue}
     */
    private class PaletteChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            if (pce.getPropertyName().equals(SensorDataObject.PROP_PALETTE_CHANGED)) {
                Palette palette = (Palette) pce.getNewValue();
                if (palette != null) {
                    layer.setPalette(palette);
                }
            }
        }
    }
}